__author__ = 'Dmitriy Korsakov'

API_KEY_ID = None

API_SECRET_KEY = None

API_VERSION = "v1beta0"

envId = None

view = "tree"

debug_mode = False

colored_output = True

API_SCHEME = 'https'

API_HOST = 'my.scalr.net'

SIGNATURE_VERSION = 'V1-HMAC-SHA256'
